@@ -2,6 +2,7 @@
  *  xfdesktop - xfce4's desktop manager
  *
  *  Copyright (c) 2004 Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright (c) 2010 Jannis Pohlmann, <jannis@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,8 +31,15 @@
 
 #include <X11/Xlib.h>
 
-#define BACKDROP_CHANNEL         "BACKDROP"
-#define DEFAULT_BACKDROP         DATADIR "/xfce4/backdrops/xfce-smoke.png"
+#define XFDESKTOP_CHANNEL        "xfce4-desktop"
+#define DEFAULT_BACKDROP         DATADIR "/backgrounds/xfce/xfce-blue.jpg"
+#define DEFAULT_BACKDROP_LIST    "xfce4/desktop/backdrop.list"
+#define DEFAULT_ICON_FONT_SIZE   12
+#define DEFAULT_ICON_SIZE        32
+#define ITHEME_FLAGS             (GTK_ICON_LOOKUP_USE_BUILTIN \
+                                  | GTK_ICON_LOOKUP_GENERIC_FALLBACK \
+                                  | GTK_ICON_LOOKUP_FORCE_SIZE)
+
 #define LIST_TEXT                "# xfce backdrop list"
 #define XFDESKTOP_SELECTION_FMT  "XFDESKTOP_SELECTION_%d"
 #define XFDESKTOP_IMAGE_FILE_FMT "XFDESKTOP_IMAGE_FILE_%d"
@@ -39,16 +47,46 @@
 #define RELOAD_MESSAGE     "reload"
 #define MENU_MESSAGE       "menu"
 #define WINDOWLIST_MESSAGE "windowlist"
+#define ARRANGE_MESSAGE    "arrange"
 #define QUIT_MESSAGE       "quit"
 
+/**
+ * File information namespaces queried for #GFileInfo objects.
+ */
+#define XFDESKTOP_FILE_INFO_NAMESPACE \
+  "access::*," \
+  "id::*," \
+  "mountable::*," \
+  "preview::*," \
+  "standard::*," \
+  "time::*," \
+  "thumbnail::*," \
+  "trash::*," \
+  "unix::*"
+
+/**
+ * Filesystem information namespaces queried for #GFileInfo * objects.
+ */
+#define XFDESKTOP_FILESYSTEM_INFO_NAMESPACE \
+  "filesystem::*"
+
 G_BEGIN_DECLS
 
-gchar **get_list_from_file(const gchar *);
-gboolean is_backdrop_list(const gchar *path);
-gboolean xfdesktop_check_image_file(const gchar *filename);
-gchar *xfce_desktop_get_menufile(void);
+gchar **xfdesktop_backdrop_list_load(const gchar *filename,
+                                     gint *n_items,
+                                     GError **error);
+gboolean xfdesktop_backdrop_list_save(const gchar *filename,
+                                      gchar * const *files,
+                                      GError **error);
+gchar *xfdesktop_backdrop_list_choose_random(const gchar *filename,
+                                             GError **error);
+gboolean xfdesktop_backdrop_list_is_valid(const gchar *filename);
+
+gboolean xfdesktop_image_file_is_valid(const gchar *filename);
+
 gboolean xfdesktop_check_is_running(Window *xid);
 void xfdesktop_send_client_message(Window xid, const gchar *msg);
+
 gboolean xfdesktop_popup_grab_available(GdkWindow *win, guint32 timestamp);
 
 G_END_DECLS
