@@ -52,6 +52,8 @@
 #include "xfdesktop-notify.h"
 #endif
 
+#include <exo/exo.h>
+
 #include "xfdesktop-common.h"
 #include "xfdesktop-file-utils.h"
 #include "xfdesktop-volume-icon.h"
@@ -224,6 +226,30 @@ xfdesktop_volume_icon_invalidate_pixbuf(XfdesktopVolumeIcon *icon)
     }
 }
 
+static gboolean
+xfdesktop_volume_icon_is_mounted(XfdesktopIcon *icon)
+{
+    GVolume *volume = NULL;
+    GMount *mount = NULL;
+    gboolean ret = FALSE;
+    XfdesktopVolumeIcon *volume_icon = XFDESKTOP_VOLUME_ICON(icon);
+
+    g_return_val_if_fail(XFDESKTOP_IS_VOLUME_ICON(icon), FALSE);
+
+    volume = xfdesktop_volume_icon_peek_volume(volume_icon);
+
+    if(volume != NULL)
+        mount = g_volume_get_mount(volume);
+
+    if(mount != NULL) {
+        ret = TRUE;
+        g_object_unref(mount);
+    } else {
+        ret = FALSE;
+    }
+
+    return ret;
+}
 
 static GdkPixbuf *
 xfdesktop_volume_icon_peek_pixbuf(XfdesktopIcon *icon,
@@ -244,6 +270,17 @@ xfdesktop_volume_icon_peek_pixbuf(XfdesktopIcon *icon,
 
         file_icon->priv->pix = xfdesktop_file_utils_get_icon(NULL, gicon, size, 
                                                              NULL, 100);
+
+        /* If the volume isn't mounted show it as semi-transparent */
+        if(!xfdesktop_volume_icon_is_mounted(icon)) {
+            GdkPixbuf *temp;
+            temp = exo_gdk_pixbuf_lucent(file_icon->priv->pix, 50);
+
+            if(temp != NULL) {
+                g_object_unref(G_OBJECT(file_icon->priv->pix));
+                file_icon->priv->pix = temp;
+            }
+        }
         
         file_icon->priv->cur_pix_size = size;
     }
@@ -268,10 +305,6 @@ xfdesktop_volume_icon_peek_label(XfdesktopIcon *icon)
 static GdkDragAction
 xfdesktop_volume_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
 {
-    XfdesktopVolumeIcon *volume_icon = XFDESKTOP_VOLUME_ICON(icon);
-    GVolume *volume;
-    GMount *mount;
-    
     /* volume icons more or less represent the volume's mount point, usually
      * (hopefully) a local path.  so when it's mounted, we certainly can't move
      * the mount point, but copying and linking should be OK.  when not mounted,
@@ -281,10 +314,7 @@ xfdesktop_volume_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
     /* FIXME: should i allow all actions if not mounted as well, and try to
      * mount and resolve on drop? */
     
-    volume = xfdesktop_volume_icon_peek_volume(volume_icon);
-
-    mount = g_volume_get_mount(volume);
-    if(mount) {
+    if(xfdesktop_volume_icon_is_mounted(icon)) {
         GFileInfo *info = xfdesktop_file_icon_peek_file_info(XFDESKTOP_FILE_ICON(icon));
         if(info) {
             if(g_file_info_get_attribute_boolean(info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ))
@@ -292,8 +322,6 @@ xfdesktop_volume_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
             else
                 return GDK_ACTION_LINK;
         }
-
-        g_object_unref(mount);
     }
     
     return 0;
@@ -302,26 +330,18 @@ xfdesktop_volume_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
 static GdkDragAction
 xfdesktop_volume_icon_get_allowed_drop_actions(XfdesktopIcon *icon)
 {
-    XfdesktopVolumeIcon *volume_icon = XFDESKTOP_VOLUME_ICON(icon);
-    GVolume *volume;
-    GMount *mount;
-    
     /* if not mounted, it doesn't really make sense to allow any operations
      * here.  if mounted, we should allow everything if it's writable. */
     
     /* FIXME: should i allow all actions if not mounted as well, and try to
      * mount and resolve on drop? */
 
-    volume = xfdesktop_volume_icon_peek_volume(volume_icon);
-
-    mount = g_volume_get_mount(volume);
-    if(mount) {
+    if(xfdesktop_volume_icon_is_mounted(icon)) {
         GFileInfo *info = xfdesktop_file_icon_peek_file_info(XFDESKTOP_FILE_ICON(icon));
         if(info) {
             if(g_file_info_get_attribute_boolean(info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))
                 return GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK;
         }
-        g_object_unref(mount);
     }
     
     return 0;
@@ -413,11 +433,15 @@ xfdesktop_volume_icon_peek_tooltip(XfdesktopIcon *icon)
 
             size = g_file_info_get_attribute_uint64(fs_info,
                                                     G_FILE_ATTRIBUTE_FILESYSTEM_SIZE);
-            size_string = g_format_size_for_display(size);
-
             free_space = g_file_info_get_attribute_uint64(fs_info,
                                                           G_FILE_ATTRIBUTE_FILESYSTEM_FREE);
+#if GLIB_CHECK_VERSION (2, 30, 0)
+            size_string = g_format_size(size);
+            free_space_string = g_format_size(free_space);
+#else
+            size_string = g_format_size_for_display(size);
             free_space_string = g_format_size_for_display(free_space);
+#endif
 
             volume_icon->priv->tooltip =
                 g_strdup_printf(_("Removable Volume\nMounted in \"%s\"\n%s left (%s total)"),
@@ -449,7 +473,7 @@ xfdesktop_volume_icon_eject_finish(GObject *object,
     g_return_if_fail(G_IS_ASYNC_RESULT(result));
     g_return_if_fail(XFDESKTOP_IS_VOLUME_ICON(icon));
 
-    if(!g_volume_eject_finish(volume, result, &error)) {
+    if(!g_volume_eject_with_operation_finish(volume, result, &error)) {
         /* ignore GIO errors handled internally */
         if(error->domain != G_IO_ERROR || error->code != G_IO_ERROR_FAILED_HANDLED) {
             gchar *volume_name = g_volume_get_name(volume);
@@ -491,7 +515,7 @@ xfdesktop_volume_icon_unmount_finish(GObject *object,
     g_return_if_fail(G_IS_ASYNC_RESULT(result));
     g_return_if_fail(XFDESKTOP_IS_VOLUME_ICON(icon));
 
-    if(!g_mount_unmount_finish(mount, result, &error)) {
+    if(!g_mount_unmount_with_operation_finish(mount, result, &error)) {
         /* ignore GIO errors handled internally */
         if(error->domain != G_IO_ERROR || error->code != G_IO_ERROR_FAILED_HANDLED) {
             gchar *mount_name = g_mount_get_name(mount);
@@ -607,18 +631,22 @@ xfdesktop_volume_icon_menu_toggle_mount(GtkWidget *widget,
 #ifdef HAVE_LIBNOTIFY
             xfdesktop_notify_eject(volume);
 #endif
-
-            g_volume_eject(volume, G_MOUNT_UNMOUNT_NONE, NULL,
-                           xfdesktop_volume_icon_eject_finish,
-                           g_object_ref(icon));
+            /* TODO: GMountOperation could be used to show what processes
+             *       are preventing an eject. */
+            g_volume_eject_with_operation(volume, G_MOUNT_UNMOUNT_NONE,
+                                          NULL, NULL,
+                                          xfdesktop_volume_icon_eject_finish,
+                                          g_object_ref(icon));
         } else {
 #ifdef HAVE_LIBNOTIFY
             xfdesktop_notify_unmount(mount);
 #endif
-
-            g_mount_unmount(mount, G_MOUNT_UNMOUNT_NONE, NULL,
-                            xfdesktop_volume_icon_unmount_finish, 
-                            g_object_ref(icon));
+            /* TODO: GMountOperation could be used to show what processes
+             *       are preventing an unmount. */
+            g_mount_unmount_with_operation(mount, G_MOUNT_UNMOUNT_NONE,
+                                           NULL, NULL,
+                                           xfdesktop_volume_icon_unmount_finish,
+                                           g_object_ref(icon));
         }
     } else {
         GMountOperation *operation;
@@ -708,7 +736,7 @@ xfdesktop_volume_icon_populate_context_menu(XfdesktopIcon *icon,
 
     img = gtk_image_new_from_stock(GTK_STOCK_PROPERTIES, GTK_ICON_SIZE_MENU);
     gtk_widget_show(img);
-    mi = gtk_image_menu_item_new_with_mnemonic(_("_Properties..."));
+    mi = gtk_image_menu_item_new_with_mnemonic(_("P_roperties..."));
     gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
     gtk_widget_show(mi);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
@@ -964,6 +992,11 @@ xfdesktop_volume_icon_new(GVolume *volume,
         g_object_unref(mount);
     }
 
+    g_signal_connect_swapped(G_OBJECT(gtk_icon_theme_get_for_screen(screen)),
+                             "changed",
+                             G_CALLBACK(xfdesktop_volume_icon_invalidate_pixbuf),
+                             volume_icon);
+
     g_signal_connect(volume, "changed", 
                      G_CALLBACK(xfdesktop_volume_icon_changed), 
                      volume_icon);
